﻿using System;
using System.Collections.Generic;
using Analyzer.Core;

namespace Analyzer.Atoms
{
    public static class BlobTest
    {
        private class Blob
            : IComparable<Blob>, IEquatable<Blob>
        {
            private int end;
            public int start;

            public Blob(int start, int end)
            {
                this.end = end;
                this.start = start;
            }

            public int CompareTo(Blob other)
            {
                return this.end - other.end;
            }

            public bool Equals(Blob other)
            {
                return this.end == other.end && this.start == other.start;
            }

            public void Intersect(Alignment a)
            {
                if (!Intersects(a))
                {
                    throw new ArgumentException();
                }
                else
                {
                    this.end = Math.Min(this.end, a.End.Position);
                    this.start = Math.Max(this.start, a.Start.Position);
                }
            }

            public void Intersect(Blob b)
            {
                if (!Intersects(b))
                {
                    throw new ArgumentException();
                }
                else
                {
                    this.end = Math.Min(this.end, b.end);
                    this.start = Math.Max(this.start, b.start);
                }
            }

            public bool Intersects(Alignment a)
            {
                return this.end >= a.Start.Position && this.start <= a.End.Position;
            }

            public bool Intersects(Blob b)
            {
                return this.end >= b.start && this.start <= b.end;
            }
        }

        private static List<Blob> Bubble(SelfAlignmentSet s, Blob start)
        {
            List<Blob> blobs = new List<Blob>();
            blobs.Add(start);
            Dictionary<List<Blob>, int> history = new Dictionary<List<Blob>, int>(new SortedListComparer<Blob>());
            int step = 0;
            while (!history.ContainsKey(blobs))
            {
                history.Add(blobs, step);
                step += 1;
                List<Blob> next = new List<Blob>(blobs);
                foreach (Alignment align in s.Alignments)
                {
                    int i = blobs.BinarySearch(new Blob(align.Start.Position, align.Start.Position));
                    if (i < 0)
                    {
                        i = ~i;
                    }
                    while (i < blobs.Count && blobs[i].Intersects(align))
                    {
                        i++;
                    }
                }
                blobs = next;
            }
            return blobs;
        }

        public static void Run(SelfAlignmentSet s)
        {
            Bubble(s, new Blob(10000, 10000));
        }
    }
}
