﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using Analyzer.Core;

namespace Analyzer.Atoms
{
    public static class MappingTest
    {
        private static void Draw(Dictionary<List<int>, int> history, SingleAlignmentSet ss)
        {
            int step = 0;
            while (step < history.Count)
            {
                foreach (KeyValuePair<List<int>, int> kvp in history)
                {
                    if (kvp.Value == step)
                    {
                        Image img = Plot.Plotter.PlotAlignment(ss, 1000);
                        Graphics g = Graphics.FromImage(img);
                        g.SmoothingMode = SmoothingMode.HighQuality;
                        g.TranslateTransform(0, img.Height);
                        g.ScaleTransform((float)img.Width / ss.Target.Length, -(float)img.Height / ss.Query.Length);
                        Brush b = new SolidBrush(Color.FromArgb(128, Color.Red));
                        foreach (int p in kvp.Key)
                        {
                            g.FillEllipse(b, p, p, 5f * ss.Target.Length / img.Width, 5f * ss.Target.Length / img.Width);
                        }
                        img.Save(step.ToString("D3") + ".png");
                        break;
                    }
                }
                step += 1;
            }
        }

        public static void Run(SelfAlignmentSet s)
        {
            List<int> points = new List<int>();
            points.Add((new Random()).Next(s.Sequence.Length));
            Dictionary<List<int>, int> history = new Dictionary<List<int>, int>(new SortedListComparer<int>());
            int step = 0;
            while (!history.ContainsKey(points))
            {
                Console.WriteLine("    step {0} -- {1} image(s)", step, points.Count);
                history.Add(points, step);
                step += 1;
                points = Step(points, s);
                points = Compactify(points, s.Sequence.Length);
            }
            Console.WriteLine("  {0} image(s) after {1} step(s) in random-point test", points.Count, step - 1);
            Save(history, @"out.txt");
            Draw(history, s);
        }

        private static void Save(Dictionary<List<int>, int> history, string path)
        {
            using (StreamWriter writer = new StreamWriter(path))
            {
                int step = 0;
                while (step < history.Count)
                {
                    foreach (KeyValuePair<List<int>, int> kvp in history)
                    {
                        if (kvp.Value == step)
                        {
                            foreach (int p in kvp.Key)
                            {
                                writer.Write("{0},", p);
                            }
                            writer.WriteLine();
                            break;
                        }
                    }
                    step += 1;
                }
            }
        }

        private static List<int> Step(List<int> points, SelfAlignmentSet alignSet)
        {
            Dictionary<int, object> dic = new Dictionary<int, object>();
            foreach (int p in points)
            {
                dic.Add(p, null);
            }
            foreach (Alignment align in alignSet.Alignments)
            {
                int i = points.BinarySearch(align.Start.Position);
                if (i < 0)
                {
                    i = ~i;
                }
                while (i < points.Count && points[i] < align.End.Position)
                {
                    // TODO: fix this line
                    //dic[align.FlatMap(points[i])] = null;
                    i++;
                }
            }
            List<int> res = new List<int>(dic.Keys);
            res.Sort();
            return res;
        }

        private const int minDist = 300;

        private static List<int> Compactify(List<int> points, int length)
        {
            int[] best = new int[length];
            int[] prev = new int[length];
            int[] output = new int[length];
            for (int i = 0; i < length; i++)
            {
                output[i] = 0;
                best[i] = int.MaxValue;
                for (int last = i - minDist; last > i - 2 * minDist; last--)
                {
                    if (last < 0)
                    {
                        int score = 0;
                        for (int j = 0; j < points.Count && points[j] < i; j++)
                        {
                            score += i - points[j];
                        }
                        if (score < best[i])
                        {
                            best[i] = score;
                            prev[i] = -1;
                        }
                    }
                    else
                    {
                        int score = best[last];
                        int j = points.BinarySearch(last);
                        if (j < 0)
                        {
                            j = ~j;
                        }
                        while (j < points.Count && points[j] < i)
                        {
                            score += Math.Min(points[j] - last, i - points[j]);
                            j += 1;
                        }
                        if (score < best[i])
                        {
                            best[i] = score;
                            prev[i] = last;
                        }
                    }
                }
            }
            int minScore = int.MaxValue;
            int end = -1;
            for (int i = length - minDist; i < length; i++)
            {
                int score = best[i];
                foreach (int p in points)
                {
                    if (p > i)
                    {
                        score += p - i;
                    }
                }
                if (score < minScore)
                {
                    minScore = score;
                    end = i;
                }
            }
            while (end != -1)
            {
                output[end] = 1;
                end = prev[end];
            }
            foreach (int p in points)
            {
                int d = 0;
                while (!(p - d >= 0 && output[p - d] > 0) && !(p + d < length && output[p + d] > 0))
                {
                    d += 1;
                }
                if (p - d >= 0 && output[p - d] > 0)
                {
                    output[p - d] = 2;
                }
                else if (p + d < length && output[p + d] > 0)
                {
                    output[p + d] = 2;
                }
            }
            List<int> res = new List<int>();
            for (int i = 0; i < length; i++)
            {
                if (output[i] == 2)
                {
                    res.Add(i);
                }
            }
            return res;
        }
    }
}
