﻿using System;

namespace Analyzer.Core
{
    public class AlignmentBlock
    {
        private readonly SequenceInterval queryInterval;
        private readonly SequenceInterval targetInterval;
        private readonly bool reversed;

        public AlignmentBlock(Sequence query, Sequence target, int queryStart, int targetStart, int length, bool reversed)
        {
            this.queryInterval = new SequenceInterval(query, queryStart, queryStart + length);
            this.targetInterval = new SequenceInterval(target, targetStart, targetStart + length);
            this.reversed = reversed;
        }

        public AlignmentBlock(SequenceInterval queryInterval, SequenceInterval targetInterval, bool reversed)
        {
            this.queryInterval = queryInterval;
            this.targetInterval = targetInterval;
            this.reversed = reversed;
        }

        public SequencePosition End
        {
            get
            {
                return queryInterval.End;
            }
        }

        public bool IsReversed
        {
            get
            {
                return reversed;
            }
        }

        public int Length
        {
            get
            {
                return queryInterval.Length;
            }
        }

        public SequencePosition Map(SequencePosition position)
        {
            if (queryInterval.Contains(position))
            {
                if (reversed)
                {
                    return targetInterval.End - (position.Position - queryInterval.Start.Position);
                }
                else
                {
                    return targetInterval.Start + (position.Position - queryInterval.Start.Position);
                }
            }
            else
            {
                throw new ArgumentOutOfRangeException("position");
            }
        }

        public SequencePosition MapBack(SequencePosition position)
        {
            if (targetInterval.Contains(position))
            {
                if (reversed)
                {
                    return queryInterval.End - (position.Position - targetInterval.Start.Position);
                }
                else
                {
                    return queryInterval.Start + (position.Position - targetInterval.Start.Position);
                }
            }
            else
            {
                throw new ArgumentOutOfRangeException("position");
            }
        }

        public SequencePosition MappedEnd
        {
            get
            {
                if (reversed)
                {
                    return targetInterval.Start;
                }
                else
                {
                    return targetInterval.End;
                }
            }
        }

        public SequencePosition MappedStart
        {
            get
            {
                if (reversed)
                {
                    return targetInterval.End;
                }
                else
                {
                    return targetInterval.Start;
                }
            }
        }

        public Sequence Query
        {
            get
            {
                return queryInterval.Sequence;
            }
        }

        public SequenceInterval QueryInterval
        {
            get
            {
                return queryInterval;
            }
        }

        public SequencePosition Start
        {
            get
            {
                return queryInterval.Start;
            }
        }

        public Sequence Target
        {
            get
            {
                return targetInterval.Sequence;
            }
        }

        public SequenceInterval TargetInterval
        {
            get
            {
                return targetInterval;
            }
        }
    }
}
