﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace Analyzer.Core
{
    class ReadOnlyDictionary<TKey, TValue>
        : IDictionary<TKey, TValue>
    {
        private readonly Dictionary<TKey, TValue> data;

        public ReadOnlyDictionary(IDictionary<TKey, TValue> dictionary)
        {
            data = new Dictionary<TKey, TValue>(dictionary);
        }

        public ReadOnlyDictionary(IDictionary<TKey, TValue> dictionary, IEqualityComparer<TKey> comparer)
        {
            data = new Dictionary<TKey, TValue>(dictionary, comparer);
        }

        void ICollection<KeyValuePair<TKey, TValue>>.Add(KeyValuePair<TKey, TValue> item)
        {
            throw new NotSupportedException();
        }

        void ICollection<KeyValuePair<TKey, TValue>>.Clear()
        {
            throw new NotSupportedException();
        }

        bool ICollection<KeyValuePair<TKey, TValue>>.Contains(KeyValuePair<TKey, TValue> item)
        {
            return ((ICollection<KeyValuePair<TKey, TValue>>)data).Contains(item);
        }

        void ICollection<KeyValuePair<TKey, TValue>>.CopyTo(KeyValuePair<TKey, TValue>[] array, int arrayIndex)
        {
            ((ICollection<KeyValuePair<TKey, TValue>>)data).CopyTo(array, arrayIndex);
        }

        bool ICollection<KeyValuePair<TKey, TValue>>.IsReadOnly
        {
            get
            {
                return true;
            }
        }

        bool ICollection<KeyValuePair<TKey, TValue>>.Remove(KeyValuePair<TKey, TValue> item)
        {
            throw new NotSupportedException();
        }

        void IDictionary<TKey, TValue>.Add(TKey key, TValue value)
        {
            throw new NotSupportedException();
        }

        bool IDictionary<TKey, TValue>.Remove(TKey key)
        {
            throw new NotSupportedException();
        }

        TValue IDictionary<TKey, TValue>.this[TKey key]
        {
            get
            {
                return data[key];
            }
            set
            {
                throw new NotSupportedException();
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable)data).GetEnumerator();
        }

        public TValue this[TKey key]
        {
            get
            {
                return ((IDictionary<TKey, TValue>)this)[key];
            }
        }

        public int Count
        {
            get
            {
                return data.Count;
            }
        }

        public bool ContainsKey(TKey key)
        {
            return data.ContainsKey(key);
        }

        public IEnumerator<KeyValuePair<TKey, TValue>> GetEnumerator()
        {
            return data.GetEnumerator();
        }

        public ICollection<TKey> Keys
        {
            get
            {
                return data.Keys;
            }
        }

        public bool TryGetValue(TKey key, out TValue value)
        {
            return data.TryGetValue(key, out value);
        }

        public ICollection<TValue> Values
        {
            get
            {
                return data.Values;
            }
        }
    }
}
