﻿using System;

namespace Analyzer.Core
{
    public class SequencePosition
        : IComparable<SequencePosition>, IEquatable<SequencePosition>
    {
        private readonly Sequence sequence;
        private readonly StrandPosition forwardPosition;
        private readonly int hashCode;

        public SequencePosition(Sequence sequence, int position)
        {
            this.sequence = sequence;
            this.forwardPosition = new StrandPosition(sequence.ForwardStrand, position);
            this.hashCode = HashProvider.Hash(forwardPosition);
        }

        public SequencePosition(Sequence sequence, StrandPosition forwardPosition)
        {
            if (forwardPosition.Strand != sequence.ForwardStrand)
            {
                throw new ArgumentException("Must be on the forward strand of sequence.", "forwardPosition");
            }
            this.sequence = sequence;
            this.forwardPosition = forwardPosition;
            this.hashCode = HashProvider.Hash(forwardPosition);
        }

        public static bool operator ==(SequencePosition x, SequencePosition y)
        {
            if ((object)x == null)
            {
                return (object)y == null;
            }
            else
            {
                return x.Equals(y);
            }
        }

        public static bool operator !=(SequencePosition x, SequencePosition y)
        {
            return !(x == y);
        }

        public static bool operator <(SequencePosition x, SequencePosition y)
        {
            return x.forwardPosition < y.forwardPosition;
        }

        public static bool operator >(SequencePosition x, SequencePosition y)
        {
            return x.forwardPosition > y.forwardPosition;
        }

        public static bool operator <=(SequencePosition x, SequencePosition y)
        {
            return x.forwardPosition <= y.forwardPosition;
        }

        public static bool operator >=(SequencePosition x, SequencePosition y)
        {
            return x.forwardPosition >= y.forwardPosition;
        }

        public static SequencePosition operator +(SequencePosition x, int y)
        {
            return new SequencePosition(x.sequence, x.forwardPosition + y);
        }

        public static SequencePosition operator -(SequencePosition x, int y)
        {
            return new SequencePosition(x.sequence, x.forwardPosition - y);
        }

        public static int operator -(SequencePosition x, SequencePosition y)
        {
            return x.forwardPosition - y.forwardPosition;
        }

        public int CompareTo(SequencePosition other)
        {
            return this.forwardPosition.CompareTo(other.forwardPosition);
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as SequencePosition);
        }

        public bool Equals(SequencePosition other)
        {
            if ((object)other == null)
            {
                return false;
            }
            else
            {
                return this.forwardPosition.Equals(other.forwardPosition);
            }
        }

        public override int GetHashCode()
        {
            return hashCode;
        }

        public int Position
        {
            get
            {
                return forwardPosition.Position;
            }
        }

        public Sequence Sequence
        {
            get
            {
                return sequence;
            }
        }

        public StrandPosition ToForwardStrand()
        {
            return forwardPosition;
        }

        public StrandPosition ToReverseStrand()
        {
            return forwardPosition.ReverseComplement;
        }
    }
}
