﻿using System;
using System.Collections.Generic;
using Analyzer.Core;

namespace Analyzer.Stats
{
    public static class MainClass
    {
        public static void Main(string[] args)
        {
            foreach (string fileName in args)
            {
                Console.WriteLine(fileName);
                MultiAlignmentSet s = MultiAlignmentSet.FromPSL(fileName);
                Console.Write("Targets: ");
                foreach (Sequence seq in s.Targets)
                {
                    Console.Write("{0} ({1} bp) ", seq.Name, seq.Length);
                }
                Console.WriteLine();
                Console.Write("Queries: ");
                foreach (Sequence seq in s.Queries)
                {
                    Console.Write("{0} ({1} bp) ", seq.Name, seq.Length);
                }
                Console.WriteLine();
                int blocks = 0;
                int gapLength = 0;
                int gaps = 0;
                int maxGap = 0;
                foreach (Alignment align in s.Alignments)
                {
                    blocks += align.Blocks.Count;
                    gaps += align.Blocks.Count - 1;
                    for (int i = 1; i < align.Blocks.Count; i++)
                    {
                        int qGap = align.Blocks[i].QueryInterval.Start - align.Blocks[i - 1].QueryInterval.End;
                        int tGap = Math.Abs(align.Blocks[i].TargetInterval.Start - align.Blocks[i - 1].TargetInterval.End);
                        gapLength += qGap + tGap;
                        maxGap = Math.Max(maxGap, qGap);
                        maxGap = Math.Max(maxGap, tGap);
                    }
                }
                Console.WriteLine("  {0} alignments", s.Alignments.Count);
                Console.WriteLine("  {0} blocks", blocks);
                Console.WriteLine("  Average gap: {0:F2}", (double)gapLength / gaps);
                Console.WriteLine("  Longest gap: {0}", maxGap);
            }
        }
    }
}
