#include <sstream>
#include "parsing.h"

using namespace std;

// Removes any trailing comments starting from the first occurence of '#'.
void remove_comments(string &line)
{
    string::size_type pos;
    pos = line.find_first_of('#');
    if (pos != string::npos)
    {
        line.erase(pos);
    }
}

// Splits a line of config file into a setting and its value.
void split(const string &line, string &setting, string &value)
{
    setting = "";
    value = "";
    istringstream ss(line);
    ss >> setting >> ws;
    getline(ss, value);
}

// Reads the next setting from a config file. Returns false if there are no
// more settings.
bool parse_line(istream &input, string &setting, string &value)
{
    string line;
    while (!input.eof())
    {
        getline(input, line);
        remove_comments(line);
        split(line, setting, value);
        if (setting != "")
        {
            trim(value);
            return true;
        }
    }
    return false;
}

// Reads the next character from input, skipping over any whitespaces. Returns
// '\0' if there are no more characters.
char read_char(istream &input)
{
    char c;
    input >> c;
    if (input.fail())
    {
        return '\0';
    }
    return c;
}

// Reads the next decimal number from input, skipping over any whitespaces. If
// this method fails, return value is undefined and input stream will have its
// error flags set.
double read_double(istream &input)
{
    double d;
    input >> d;
    return d;
}

// Reads the next integer from input, skipping over any whitespaces. If this
// method fails, return value is undefined and input stream will have its error
// flags set.
int read_int(istream &input)
{
    int i;
    input >> i;
    return i;
}

// Reads a line of the input stream. If this method fails, return value is
// undefined and input stream will have its error flags set.
string read_line(istream &input)
{
    string line;
    getline(input, line);
    return line;
}

// Removes leading and trailing spaces.
void trim(string &str)
{
    string::size_type pos;
    pos = str.find_last_not_of(' ');
    if (pos != string::npos)
    {
        str.erase(pos + 1);
        pos = str.find_first_not_of(' ');
        if (pos != string::npos)
        {
            str.erase(0, pos);
        }
    }
    else
    {
        str.clear();
    }
}
