#include <sstream>
#include "parsing.h"
#include "phyltree.h"

using namespace std;

// Creates a deep copy of the subtree.
phyltree::phyltree(const phyltree &t)
{
    if (t.left != NULL)
    {
        left = new phyltree(*t.left);
    }
    else
    {
        left = NULL;
    }
    if (t.right != NULL)
    {
        right = new phyltree(*t.right);
    }
    else
    {
        right = NULL;
    }
    parent = NULL;
    left_length = t.left_length;
    right_length = t.right_length;
    species = t.species;
}

// Creates a deep copy of the subtree.
phyltree& phyltree::operator=(const phyltree &t)
{
    if (this != &t)
    {
        cleanup();
        if (t.left != NULL)
        {
            left = new phyltree(*t.left);
        }
        else
        {
            left = NULL;
        }
        if (t.right != NULL)
        {
            right = new phyltree(*t.right);
        }
        else
        {
            right = NULL;
        }
        left_length = t.left_length;
        right_length = t.right_length;
        species = t.species;
    }
    return *this;
}

phyltree::phyltree(istream &input, int &counter)
{
    parse(input, counter);
}

phyltree::phyltree(const string &description)
{
    istringstream ss(description);
    int counter = 1;
    parse(ss, counter);
    ss >> ws;
    if (!ss.eof())
    {
        cleanup();
        throw (string)"Invalid tree description, nothing more expected.";
    }
}

phyltree::~phyltree()
{
    cleanup();
}

void phyltree::cleanup()
{
    if (left != NULL)
    {
        delete left;
    }
    if (right != NULL)
    {
        delete right;
    }
}

double read_length(istream &input)
{
    double d = read_double(input);
    if (input.fail())
    {
        throw (string)"Invalid tree description, number expected.";
    }
    return d;
}

// Consumes the given character skipping over any whitespaces.
void consume(istream &input, char c)
{
    if (read_char(input) != c)
    {
        throw (string)"Invalid tree description, expected '" + c + "'.";
    }
}

void phyltree::parse(istream &input, int &counter)
{
    left = NULL;
    right = NULL;
    try
    {
        switch (read_char(input))
        {
            case '(':
                left = new phyltree(input, counter);
                left_length = read_length(input);
                left->parent = this;
                left->parent_length = left_length;
                consume(input, ',');
                right = new phyltree(input, counter);
                right_length = read_length(input);
                right->parent = this;
                right->parent_length = right_length;
                consume(input, ')');
                break;
            default:
                input.unget();
                break;
        }
        getline(input, species, ':');
        trim(species);
        if (species == "")
        {
            if (left == NULL)
            {
                stringstream ss;
                ss << "species_";
                ss << counter;
                species = ss.str();
                counter += 1;
            }
            else
            {
                species = left->species + "-" + right->species;
            }
        }
    }
    catch (string ex)
    {
        cleanup();
        throw;
    }
}

string phyltree::to_string()
{
    if (left == NULL)
    {
        return species;
    }
    else
    {
        ostringstream ss;
        ss << "(" << left->to_string() << ":" << left_length << ", " << right->to_string() << ":" << right_length << ")" << species;
        return ss.str();
    }
}
