#ifndef DISTRIBUTION_H
#define DISTRIBUTION_H

#include <string>

class distribution
{
    public:
        static distribution* from_string(const std::string &description);
        distribution(double min, double max);
        virtual distribution* clone() const = 0;
        double sample();
        int sample_int();
        
    protected:
        virtual double next() = 0;
        
    private:
        double min;
        double max;
};

class constdist : public distribution
{
    public:
        constdist(double value);
        distribution* clone() const;
        
    protected:
        double next();
        
    private:
        double value;
};

class expdist : public distribution
{
    public:
        expdist(double lambda);
        expdist(double lambda, double min, double max);
        distribution* clone() const;
        
    protected:
        double next();
        
    private:
        double lambda;
        
        void init(double lambda);
};

class geodist : public distribution
{
    public:
        geodist(double p);
        geodist(double p, double min, double max);
        distribution* clone() const;
        
    protected:
        double next();
        
    private:
        double log_q;
        
        void init(double p);
};

class negbindist : public distribution
{
    public:
        negbindist(int r, double p);
        negbindist(int r, double p, double min, double max);
        distribution* clone() const;
        
    protected:
        double next();
        
    private:
        int r;
        geodist g;
        
        void init(double r);
};

class zipfdist : public distribution
{
    public:
        zipfdist(double a);
        zipfdist(double a, double min, double max);
        distribution* clone() const;
        
    protected:
        double next();
        
    private:
        double a;
        double b;
        double c;
        
        void init(double a);
};

#endif
