#ifndef SEQEVENT_H
#define SEQEVENT_H

#include <ostream>
#include <vector>
#include "distribution.h"

class seqevent
{
    public:
        static seqevent* from_string(const std::string &description);
        seqevent(const std::string &name, double rate);
        virtual ~seqevent();
        virtual seqevent* clone() const = 0;
        virtual bool needs_substitutions() = 0;
        double next_occurence(int seqlen);
        virtual void simulate(std::vector<char> &seq, double base_freqs[4], std::ostream &logfile);
        void wait(double time, int seqlen);
        
    protected:
        std::string name;
        expdist time;
        double next_time;
        
        virtual int affecting_positions(int seqlen) = 0;
};

class insertion : public seqevent
{
    public:
        insertion(const std::string &name, double rate, const distribution &length);
        insertion(const insertion &ins);
        insertion& operator=(const insertion &ins);
        ~insertion();
        seqevent* clone() const;
        bool needs_substitutions();
        void simulate(std::vector<char> &seq, double base_freqs[4], std::ostream &logfile);
        
    protected:
        int affecting_positions(int seqlen);
        
    private:
        distribution *length;
};

class deletion : public seqevent
{
    public:
        deletion(const std::string &name, double rate, const distribution &length);
        deletion(const deletion &del);
        deletion& operator=(const deletion &del);
        ~deletion();
        seqevent* clone() const;
        bool needs_substitutions();
        void simulate(std::vector<char> &seq, double base_freqs[4], std::ostream &logfile);
        
    protected:
        int affecting_positions(int seqlen);
        
    private:
        distribution *length;
        int next_length;
};

class duplication : public seqevent
{
    public:
        duplication(const std::string &name, double rate, const distribution &length, const distribution &distance, double inv_rate);
        duplication(const duplication &dup);
        duplication& operator=(const duplication &dup);
        ~duplication();
        seqevent* clone() const;
        bool needs_substitutions();
        void simulate(std::vector<char> &seq, double base_freqs[4], std::ostream &logfile);
        
    protected:
        int affecting_positions(int seqlen);
        
    private:
        distribution *length;
        distribution *dist;
        double inv_rate;
};

#endif
