#include "random.h"
#include "sequence.h"

using namespace std;

// Converts a base to its integer representation.
int btoi(char base)
{
    switch (base)
    {
        case 'T':
            return 0;
        case 'C':
            return 1;
        case 'A':
            return 2;
        case 'G':
            return 3;
        default:
            throw (string)"Internal error in sequence.cpp: btoi(char base).";
    }
}

// Converts an integer representation of base to the original base.
char itob(int index)
{
    switch (index)
    {
        case 0:
            return 'T';
        case 1:
            return 'C';
        case 2:
            return 'A';
        case 3:
            return 'G';
        default:
            throw (string)"Internal error in simulator.cpp: itob(int index).";
    }
}

// Retuns the complementary base.
char compl_base(char base)
{
    switch (base)
    {
        case 'T':
            return 'A';
        case 'C':
            return 'G';
        case 'A':
            return 'T';
        case 'G':
            return 'C';
        default:
            throw (string)"Internal error in simulator.cpp: compl_base(char base).";
    }
}

// Returns a random index. Array prob specifies the probabilities of each index.
// The probabilities should sum to 1.
int rand_index(double prob[4])
{
    double rnd = random::next();
    for (int i = 0; i < 4; i++)
    {
        rnd -= prob[i];
        if (rnd < 0)
        {
            return i;
        }
    }
    return 0;
}

// Returns a random base. Array base_freqs should sum to one.
char rand_base(double base_freqs[4])
{
    return itob(rand_index(base_freqs));
}

// Fills seq with a random sequence of given length. Array base_freqs should sum
// to one.
void rand_seq(int length, vector<char> &seq, double base_freqs[4])
{
    seq.clear();
    seq.reserve(length);
    for (int i = 0; i < length; i++)
    {
        seq.push_back(rand_base(base_freqs));
    }
}

// Simulates single base substitutions using the specified matrix P(t).
void subst(vector<char> &seq, double pmat[4][4])
{
    for (unsigned int i = 0; i < seq.size(); i++)
    {
        int row = btoi(seq[i]);
        int j = rand_index(pmat[row]);
        seq[i] = itob(j);
    }
}

// Saves the sequence in a fasta format.
void write_fasta(ostream &output, const vector<char> &seq, const string &name)
{
    int pos = 0;
    output << ">" << name << endl;
    while (pos + 60 < (int)seq.size())
    {
        output.write(&seq[pos], 60);
        pos += 60;
        output << endl;
    }
    output.write(&seq[pos], seq.size() - pos);
    output << endl;
}
